@@ -1,8 +1,9 @@
-/* Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements. 
- * The ASF licenses this file to You under the Apache License, Version 2.0
- * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
+/**
+ * Copyright 2006 Christian Liesch
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -136,46 +137,10 @@
 #define HTNTLM_STR_RESP_NTLM2      "ntlm2" 
 #define HTNTLM_STR_RESP_NTLM2_SESS "ntlm2-session" 
 
-#define swap16(x) \
-  ((((x) & 0x00ffU) << 8)| \
-   (((x) & 0xff00U) >> 8))
-
-#define swap32(x) \
-  ((((x) & 0x000000ffUL) << 24)| \
-   (((x) & 0x0000ff00UL) <<  8)| \
-   (((x) & 0x00ff0000UL) >>  8)| \
-   (((x) & 0xff000000UL) >> 24))
-
-#define swap64(x) \
-  ((((x) & 0xff00000000000000ULL) >> 56)| \
-   (((x) & 0x00ff000000000000ULL) >> 40)| \
-   (((x) & 0x0000ff0000000000ULL) >> 24)| \
-   (((x) & 0x000000ff00000000ULL) >>  8)| \
-   (((x) & 0x00000000ff000000ULL) <<  8)| \
-   (((x) & 0x0000000000ff0000ULL) << 24)| \
-   (((x) & 0x000000000000ff00ULL) << 40)| \
-   (((x) & 0x00000000000000ffULL) << 56))
-
-#if APR_IS_BIGENDIAN
-# define hton16(x) swap16(x)
-# define hton32(x) swap32(x)
-# define hton64(x) swap64(x)
-# define ntoh16(x) swap16(x)
-# define ntoh32(x) swap32(x)
-# define ntoh64(x) swap64(x)
-#else
-# define hton16(x) (x)
-# define hton32(x) (x)
-# define hton64(x) (x)
-# define ntoh16(x) (x)
-# define ntoh32(x) (x)
-# define ntoh64(x) (x)
-#endif
-
 #ifdef WIN32
 #define FMT_LLX "%I64x"
 #else
-#define FMT_LLX "%llx"
+#define FMT_LLX "%"APR_UINT64_T_HEX_FMT
 #endif
 
 /************************************************************************
@@ -718,7 +683,7 @@ static unsigned char * get_lm2_hash(htntlm_t *hook, uint16_t *hash_len) {
   const EVP_MD *md5 = EVP_md5();
   MD4_CTX MD4;
   char *passwd; 
-  apr_size_t len;
+  unsigned int len;
   char *uuser;
   apr_size_t uuser_len;
   char *udomain;
@@ -789,7 +754,7 @@ static unsigned char * get_ntlm2_hash(htntlm_t *hook, uint16_t *hash_len) {
   unsigned char ntlm_hash[16];
   unsigned char ntlm2_hash[16];
   unsigned char blob_hash[16];
-  apr_size_t len;
+  unsigned int len;
   unsigned char *blob;
   unsigned char *buf;
   unsigned char *target_info = NULL;
@@ -826,7 +791,6 @@ static unsigned char * get_ntlm2_hash(htntlm_t *hook, uint16_t *hash_len) {
 
   /* 3. blob */
   blob = apr_pcalloc(hook->pool, 28 + ti_len + 4);
-  //*((uint32_t *)&blob[0]) = hton32(0x01010000);
   *((uint32_t *)&blob[0]) = hton32(0x00000101);
   *((uint32_t *)&blob[4]) = hton32(0x00000000);
 #if defined(WIN32)
@@ -1314,7 +1278,6 @@ static void write_message(htntlm_t *hook) {
  */
 static void read_type1_msg(htntlm_t *hook, unsigned char *msg, int msg_len) {
   int len;
-  int alloc;
   int offset;
 
   hook->flags = ntoh32(*((uint32_t *)&msg[12]));
@@ -1325,7 +1288,6 @@ static void read_type1_msg(htntlm_t *hook, unsigned char *msg, int msg_len) {
   }
   /* domain */
   len = ntoh16(*((uint16_t *)&msg[16]));
-  alloc = ntoh16(*((uint16_t *)&msg[18]));
   offset = ntoh32(*((uint32_t *)&msg[20]));
   if (len) {
     hook->domain = apr_pstrndup(hook->pool, (char *)&msg[offset], len);
@@ -1336,7 +1298,6 @@ static void read_type1_msg(htntlm_t *hook, unsigned char *msg, int msg_len) {
   }
   /* workstation */
   len = ntoh16(*((uint16_t *)&msg[24]));
-  alloc = ntoh16(*((uint16_t *)&msg[26]));
   offset = ntoh32(*((uint32_t *)&msg[28]));
   if (len) {
     hook->workstation = apr_pstrndup(hook->pool, (char *)&msg[offset], len);
@@ -1352,7 +1313,6 @@ static void read_type1_msg(htntlm_t *hook, unsigned char *msg, int msg_len) {
  */
 static void read_type2_msg(htntlm_t *hook, unsigned char *msg, int msg_len) {
   uint16_t len;
-  uint16_t alloc;
   uint32_t offset;
   uint16_t subblk;
   char *b64msg;
@@ -1360,7 +1320,6 @@ static void read_type2_msg(htntlm_t *hook, unsigned char *msg, int msg_len) {
 
   /* target */
   len = ntoh16(*((uint16_t *)&msg[12]));
-  alloc = ntoh16(*((uint16_t *)&msg[14]));
   offset = ntoh32(*((uint32_t *)&msg[16]));
   if (len) {
     hook->target = handle_oem(hook, (char *)&msg[offset], len);
@@ -1385,7 +1344,6 @@ static void read_type2_msg(htntlm_t *hook, unsigned char *msg, int msg_len) {
   }
   /* target info */
   len = ntoh16(*((uint16_t *)&msg[40]));
-  alloc = ntoh16(*((uint16_t *)&msg[42]));
   offset = ntoh32(*((uint32_t *)&msg[44]));
   if (len) {
     b64len = apr_base64_encode_len(len);
@@ -1428,12 +1386,10 @@ static void read_type2_msg(htntlm_t *hook, unsigned char *msg, int msg_len) {
  */
 static void read_type3_msg(htntlm_t *hook, unsigned char *msg, int msg_len) {
   int len;
-  int alloc;
   int offset;
 
   /* lm/lmv2 response */
   len = ntoh16(*((uint16_t *)&msg[12]));
-  alloc = ntoh16(*((uint16_t *)&msg[14]));
   offset = ntoh32(*((uint32_t *)&msg[16]));
   if (len) {
     hook->lm.hash = apr_pcalloc(hook->pool, len); 
@@ -1442,7 +1398,6 @@ static void read_type3_msg(htntlm_t *hook, unsigned char *msg, int msg_len) {
   }
   /* ntlm/ntlmv2 response */
   len = ntoh16(*((uint16_t *)&msg[20]));
-  alloc = ntoh16(*((uint16_t *)&msg[22]));
   offset = ntoh32(*((uint32_t *)&msg[24]));
   if (len) {
     hook->ntlm.hash = apr_pcalloc(hook->pool, len); 
@@ -1451,21 +1406,18 @@ static void read_type3_msg(htntlm_t *hook, unsigned char *msg, int msg_len) {
   }
   /* domain */
   len = ntoh16(*((uint16_t *)&msg[28]));
-  alloc = ntoh16(*((uint16_t *)&msg[30]));
   offset = ntoh32(*((uint32_t *)&msg[32]));
   if (len) {
     hook->domain = handle_oem(hook, (char *)&msg[offset], len);
   }
   /* user */
   len = ntoh16(*((uint16_t *)&msg[36]));
-  alloc = ntoh16(*((uint16_t *)&msg[38]));
   offset = ntoh32(*((uint32_t *)&msg[40]));
   if (len) {
     hook->user = handle_oem(hook, (char *)&msg[offset], len);
   }
   /* workstation */
   len = ntoh16(*((uint16_t *)&msg[44]));
-  alloc = ntoh16(*((uint16_t *)&msg[46]));
   offset = ntoh32(*((uint32_t *)&msg[48]));
   if (len) {
     hook->workstation = handle_oem(hook, (char *)&msg[offset], len);
