@@ -1,3 +1,18 @@
+/**
+ * Copyright 2006 Christian Liesch
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 /* contributor license agreements. 
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
@@ -59,6 +74,11 @@
 /************************************************************************
  * Typedefs 
  ***********************************************************************/
+typedef struct connection_s {
+  const char *cmd;
+  apr_socket_t *socket;
+} connection_t;
+
 typedef struct stream_s {
   apr_pool_t *pool;
   apr_file_t *file;
@@ -76,6 +96,8 @@ apr_getopt_option_t options[] = {
   { "help", 'h', 0, "Display usage information (this message)" },
   { "port", 'p', 1, "Port" },
   { "command", 'e', 1, "Remote controlled command" },
+  { "restart", 'r', 0, "Restart after command terminates" },
+  { "thread", 't', 0, "Start a thread every connect and start command" },
   { NULL, 0, 0, NULL }
 };
 
@@ -105,7 +127,7 @@ static void usage(const char *progname) {
   }
   fprintf(stdout, "\n");
   fprintf(stdout, "\nExample: \n"
-          "%s -p 8080 -c \"./httproxy -p 8888 -d foo\"\n", progname);
+          "%s -p 8080 -e \"./httproxy -p 8888 -d foo\"\n", progname);
 }
 
 /**
@@ -141,6 +163,7 @@ static void * APR_THREAD_FUNC in_stream(apr_thread_t *self, void *streamv) {
   apr_file_close(stream->file);
   
   apr_thread_exit(self, status);
+  apr_pool_destroy(stream->pool);
   return 0;
 }
 
@@ -189,6 +212,7 @@ static void * APR_THREAD_FUNC out_stream(apr_thread_t *self, void *streamv) {
   }
 
   apr_thread_exit(self, status);
+  apr_pool_destroy(stream->pool);
   return 0;
 }
 
@@ -202,11 +226,14 @@ static void * APR_THREAD_FUNC out_stream(apr_thread_t *self, void *streamv) {
  *
  * @return apr status
  */
-static apr_status_t create_in_stream(apr_pool_t *pool, apr_socket_t *socket,
-                                     apr_file_t *in, apr_thread_t **thread) {
+static apr_status_t create_in_stream(apr_socket_t *socket, apr_file_t *in, 
+                                     apr_thread_t **thread) {
   apr_status_t status;
   apr_threadattr_t *tattr;
   stream_t *stream;
+  apr_pool_t *pool;
+
+  apr_pool_create(&pool, NULL);
 
   if ((status = apr_threadattr_create(&tattr, pool)) != APR_SUCCESS) {
     return status;
@@ -243,12 +270,14 @@ static apr_status_t create_in_stream(apr_pool_t *pool, apr_socket_t *socket,
  *
  * @return apr status
  */
-static apr_status_t create_out_stream(apr_pool_t *pool, apr_socket_t *socket,
-                                      apr_file_t *out, apr_thread_t **thread) {
+static apr_status_t create_out_stream(apr_socket_t *socket, apr_file_t *out, 
+                                      apr_thread_t **thread) {
   apr_status_t status;
   apr_threadattr_t *tattr;
   stream_t *stream;
+  apr_pool_t *pool;
 
+  apr_pool_create(&pool, NULL);
   if ((status = apr_threadattr_create(&tattr, pool)) != APR_SUCCESS) {
     return status;
   }
@@ -345,6 +374,55 @@ static apr_status_t exec(apr_pool_t *pool, const char *cmd, apr_proc_t *proc) {
   return APR_SUCCESS;
 }
 
+static void * APR_THREAD_FUNC handle_connection(apr_thread_t * thread, void *selfv) {
+  apr_status_t status;
+  apr_proc_t proc;
+  apr_thread_t *thread1;
+  apr_thread_t *thread2;
+  apr_thread_t *thread3;
+  apr_exit_why_e exitwhy;
+  int exitcode = 0;
+  apr_pool_t *pool;
+  connection_t *conn = selfv;
+
+  apr_pool_create(&pool, NULL);
+  fprintf(stdout, "\nExec %s", conn->cmd);
+  exec(pool, conn->cmd, &proc);
+  
+  fprintf(stdout, "\nStart threads");
+  /* start 3 threads one for in one for out and one for err */
+  if ((status = create_in_stream(conn->socket, proc.in, &thread1)) 
+      != APR_SUCCESS) {
+    fprintf(stderr, "\nERROR %s\n", my_status_str(pool, status));
+    goto error;
+  }
+  if ((status = create_out_stream(conn->socket, proc.out, &thread2))
+      != APR_SUCCESS) {
+    fprintf(stderr, "%s", my_status_str(pool, status));
+    goto error;
+  }
+  if ((status = create_out_stream(conn->socket, proc.err, &thread3))
+      != APR_SUCCESS) {
+    fprintf(stderr, "\nERROR %s\n", my_status_str(pool, status));
+    goto error;
+  }
+  
+  fprintf(stdout, "\nJoin command %s", conn->cmd);
+  fflush(stdout);
+  apr_proc_wait(&proc, &exitcode, &exitwhy, APR_WAIT);
+  if (exitcode) {
+    fprintf(stdout, "\nERROR %d", exitcode);
+    goto error;
+  }
+
+  fprintf(stdout, "\n--normal end\n");
+
+error:
+  apr_pool_destroy(pool);
+  return NULL;
+}
+
+
 /** 
  * call remote controler 
  *
@@ -362,12 +440,12 @@ int main(int argc, const char *const argv[]) {
   apr_socket_t *socket;
   apr_socket_t *listener;
   apr_sockaddr_t *local_addr;
-  apr_exit_why_e exitwhy;
-  int exitcode = 0;
-  apr_proc_t proc;
-  apr_thread_t *thread1;
-  apr_thread_t *thread2;
-  apr_thread_t *thread3;
+#define TYPE_NONE 0
+#define TYPE_RESTART 1
+#define TYPE_THREADED 2
+  int type = TYPE_NONE;
+  int count = 1;
+  int i;
 
   int port = 8080;
   const char *cmd = "";
@@ -398,6 +476,14 @@ int main(int argc, const char *const argv[]) {
     case 'e':
       cmd = optarg;
       break;
+    case 'r':
+      type = TYPE_RESTART;
+      count = -1;
+      break;
+    case 't':
+      type = TYPE_THREADED;
+      count = -1;
+      break;
     }
   }
 
@@ -438,48 +524,28 @@ int main(int argc, const char *const argv[]) {
     return status;
   }
 
-  fprintf(stdout, "\nWait for connection");
-  if ((status = apr_socket_accept(&socket, listener, pool)) != APR_SUCCESS) {
-    fprintf(stderr, "\nERROR %s\n", my_status_str(pool, status));
-    return status;
+  if (type != TYPE_THREADED) {
+    for (i = 0; i != count; i++) {
+      connection_t conn;
+
+      fprintf(stdout, "\nWait for connection");
+      if ((status = apr_socket_accept(&socket, listener, pool)) != APR_SUCCESS) {
+        fprintf(stderr, "\nERROR %s(%d)\n", my_status_str(pool, status), status);
+        return status;
+      }
+
+      conn.cmd = cmd;
+      conn.socket = socket;
+
+      handle_connection(NULL, &conn);
+    }
+  }
+  else {
+    fprintf(stderr, "\nThreaded mode is not implemented yet.\n");
   }
 
-  fprintf(stdout, "\nExec %s", cmd);
-  exec(pool, cmd, &proc);
-  
-  fprintf(stdout, "\nStart threads");
-  /* start 3 threads one for in one for out and one for err */
-  if ((status = create_in_stream(pool, socket, proc.in, &thread1)) 
-      != APR_SUCCESS) {
-    fprintf(stderr, "\nERROR %s\n", my_status_str(pool, status));
-    return exitcode;
-  }
-  if ((status = create_out_stream(pool, socket, proc.out, &thread2))
-      != APR_SUCCESS) {
-    fprintf(stderr, "%s", my_status_str(pool, status));
-    return exitcode;
-  }
-  if ((status = create_out_stream(pool, socket, proc.err, &thread3))
-      != APR_SUCCESS) {
-    fprintf(stderr, "\nERROR %s\n", my_status_str(pool, status));
-    return exitcode;
-  }
-  
-  //fprintf(stdout, "\nJoin threads");
-  //apr_thread_join(&status, thread1);
-  //apr_thread_join(&status, thread2);
-  //apr_thread_join(&status, thread3);
-
-  fprintf(stdout, "\nJoin command %s", cmd);
+  fprintf(stdout, "\nTerminated");
   fflush(stdout);
-  apr_proc_wait(&proc, &exitcode, &exitwhy, APR_WAIT);
-  if (exitcode) {
-    fprintf(stdout, "\nERROR %d", exitcode);
-    return exitcode;
-  }
-
-  fprintf(stdout, "\n--normal end\n");
-
   return 0;
 }
 
