@@ -114,6 +114,10 @@ static apr_status_t command_ERROR(command_t *self, worker_t *worker,
 
 static apr_status_t global_GO(command_t *self, global_t *global, 
 			     char *data, apr_pool_t *ptmp); 
+static apr_status_t global_START(command_t *self, global_t *global, 
+			         char *data, apr_pool_t *ptmp); 
+static apr_status_t global_JOIN(command_t *self, global_t *global, 
+			        char *data, apr_pool_t *ptmp); 
 static apr_status_t global_END(command_t *self, global_t *global, 
 			      char *data, apr_pool_t *ptmp); 
 static apr_status_t global_DAEMON(command_t *self, global_t *global, 
@@ -150,7 +154,15 @@ command_t global_commands[] = {
   "Close CLIENT|SERVER body",
   COMMAND_FLAGS_NONE},
   {"GO", (command_f )global_GO, "", 
-  "Starts all defined clients, servers and daemons. All started client and servers will be joined",
+  "Starts all defined clients, servers and daemons. "
+  "All started client and servers will be joined. "
+  "It is actually a START followed by JOIN.",
+  COMMAND_FLAGS_NONE},
+  {"START", (command_f )global_START, "", 
+  "Starts all defined clients, servers and daemons.",
+  COMMAND_FLAGS_NONE},
+  {"JOIN", (command_f )global_JOIN, "", 
+  "All started client and servers will be joined, only makes sense after START.",
   COMMAND_FLAGS_NONE},
   {"CLIENT", (command_f )global_CLIENT, "[<number of concurrent clients>]", 
   "Client body start, close it with END and a newline",
@@ -401,38 +413,41 @@ command_t local_commands[] = {
 
   /* body section */
   {"_IF", (command_f )command_IF, "(\"<string>\" [NOT] MATCH \"regex\")|(\"<number>\" [NOT] EQ|LT|GT|LE|GT \"<number>)\"|\"(\"expression\")\"", 
-   "Test string match, number equality or simply an expression to run body, close body with _END IF,\n"
+   "Test string match, number equality or simply an expression to run body, \n"
+   "close body with _END,\n"
    "negation with a leading '!' in the <regex>",
    COMMAND_FLAGS_BODY},
-  {"_LOOP", (command_f )command_LOOP, "<n>", 
-  "Do loop the body <n> times,\n"
-  "close body with _END LOOP",
+  {"_LOOP", (command_f )command_LOOP, "<n>[s|ms]|FOREVER [<variable>]", 
+  "LOOP for specified times or optional for a duration given as \"s\" or "
+  "\"ms\" with no space after number, additional you can specify a variable "
+  "which holds the loop count,\n"
+  "close body with _END",
   COMMAND_FLAGS_BODY},
   {"_FOR", (command_f )command_FOR, "<variable> \"|'<string>*\"|'", 
   "Do for each element,\n"
-  "close body with _END FOR",
+  "close body with _END",
   COMMAND_FLAGS_BODY},
   {"_BPS", (command_f )command_BPS, "<n> <duration>", 
   "Send not more than defined bytes per second, while defined duration [s]\n"
-  "close body with _END BPS",
+  "close body with _END",
   COMMAND_FLAGS_BODY},
   {"_RPS", (command_f )command_RPS, "<n> <duration>", 
   "Send not more than defined requests per second, while defined duration [s]\n"
   "Request is count on every _WAIT call\n"
-  "close body with _END RPS",
+  "close body with _END",
   COMMAND_FLAGS_BODY},
   {"_SOCKET", (command_f )command_SOCKET, "", 
   "Spawns a socket reader over the next _WAIT _RECV commands\n"
-  "close body with _END SOCKET",
+  "close body with _END",
   COMMAND_FLAGS_BODY},
   {"_ERROR", (command_f )command_ERROR, "", 
   "We do expect specific error on body exit\n"
-  "close body with _END ERROR",
+  "close body with _END",
   COMMAND_FLAGS_BODY},
 #if APR_HAS_FORK
   {"_PROCESS", (command_f )command_PROCESS, "<name>", 
   "Fork a process to run body in. Process termination handling see _PROC_WAIT\n"
-  "close body with _END PROCESS",
+  "close body with _END",
   COMMAND_FLAGS_BODY},
 #endif
 
@@ -936,8 +951,10 @@ static apr_status_t command_LOOP(command_t *self, worker_t *worker,
   char **argv;
   int i;
   char *var;
+  apr_time_t duration = 0;
+  apr_time_t start;
 
-  COMMAND_NEED_ARG("<number>|FOREVER"); 
+  COMMAND_NEED_ARG("<number>[s|ms]|FOREVER"); 
  
   my_tokenize_to_argv(copy, &argv, ptmp, 0);
 
@@ -948,7 +965,21 @@ static apr_status_t command_LOOP(command_t *self, worker_t *worker,
     loop = apr_atoi64(argv[0]);
   }
 
-  var = argv[1]; 
+  if (argv[1] != NULL) {
+    if (strcmp(argv[1], "[ms]") == 0) {
+      /* this are miliseconds we wanna loop */
+      /* apr_time_from_msec available in apr 1.4.x */
+      duration = 1000 * loop;
+      loop = -1;
+      var = argv[2]; 
+    }
+    else {
+      var = argv[1]; 
+    }
+  }
+  else {
+    var = NULL;
+  }
   
   /* create a new worker body */
   if ((status = worker_body(&body, worker)) != APR_SUCCESS) {
@@ -956,6 +987,7 @@ static apr_status_t command_LOOP(command_t *self, worker_t *worker,
   }
   
   /* loop */
+  start = apr_time_now();
   for (i = 0; loop == -1 || i < loop; i++) {
     /* interpret */
     if (var) {
@@ -964,6 +996,9 @@ static apr_status_t command_LOOP(command_t *self, worker_t *worker,
     if ((status = body->interpret(body, worker, NULL)) != APR_SUCCESS) {
       break;
     }
+    if (duration != 0 && apr_time_now() - start >= duration) {
+      break;
+    }
   }
   
   /* special case to break the loop */
@@ -1077,18 +1112,20 @@ static apr_status_t command_BPS(command_t *self, worker_t *worker, char *data,
     }
     cur = apr_time_now();
 
-    /* avoid division by zero, do happen on windows */
-    while ((cur - start == 0)) {
-      /* wait 1 ms */
-      apr_sleep(1000);
-      cur = apr_time_now();
-    }
-    
-    /* wait loop until we are below the max bps */
-    while (((body->sent * APR_USEC_PER_SEC) / (cur - start)) > bps ) {
-      /* wait 1 ms */
-      apr_sleep(1000);
-      cur = apr_time_now();
+    if (bps > 0) {
+      /* avoid division by zero, do happen on windows */
+      while ((cur - start == 0)) {
+        /* wait 1 ms */
+        apr_sleep(1000);
+        cur = apr_time_now();
+      }
+      
+      /* wait loop until we are below the max bps */
+      while (((body->sent * APR_USEC_PER_SEC) / (cur - start)) > bps ) {
+        /* wait 1 ms */
+        apr_sleep(1000);
+        cur = apr_time_now();
+      }
     }
 
     /* reset sent bytes */
@@ -1150,22 +1187,24 @@ static apr_status_t command_RPS(command_t *self, worker_t *worker, char *data,
     }
     cur = apr_time_now();
 
-    /* avoid division by zero, do happen on windows */
-    while ((cur - start == 0)) {
-      /* wait 1 ms */
-      apr_sleep(1000);
-      cur = apr_time_now();
-    }
-    
-    /* wait loop until we are below the max rps */
-    while (((body->req_cnt * APR_USEC_PER_SEC) / (cur - start)) > rps ) {
-      /* wait 1 ms */
-      apr_sleep(1000);
-      cur = apr_time_now();
-    }
+    if (rps > 0) {
+      /* avoid division by zero, do happen on windows */
+      while ((cur - start == 0)) {
+        /* wait 1 ms */
+        apr_sleep(1000);
+        cur = apr_time_now();
+      }
+      
+      /* wait loop until we are below the max rps */
+      while (((body->req_cnt * APR_USEC_PER_SEC) / (cur - start)) > rps ) {
+        /* wait 1 ms */
+        apr_sleep(1000);
+        cur = apr_time_now();
+      }
 
-    /* reset sent bytes */
-    body->req_cnt = 0;
+      /* reset sent requests */
+      body->req_cnt = 0;
+    }
 
     /* test termination */
     if (apr_time_sec(cur - init) >= duration) {
@@ -1387,7 +1426,7 @@ static apr_status_t worker_interpret(worker_t * worker, worker_t *parent,
     char *line;
     apr_pool_t *ptmp;
 
-    apr_pool_create(&ptmp, worker->pbody);
+    apr_pool_create(&ptmp, NULL);
     worker->file_and_line = e[worker->cmd].key;
     line = e[worker->cmd].val;
     if (worker_is_block(worker, line, ptmp)) {
@@ -1890,6 +1929,14 @@ static apr_status_t global_new(global_t **global, store_t *vars,
   (*global)->socktmo = 300000000;
   (*global)->prefix = apr_pstrdup(p, "");
 
+  if ((status = worker_new(&(*global)->worker, NULL, (*global)->prefix, 
+                           (*global), NULL)) != APR_SUCCESS) {
+    fprintf(stderr, "\nGlobal worker: could not create my worker");
+  }
+
+  (*global)->worker->modules = (*global)->modules;
+  (*global)->worker->name = apr_pstrdup(p, "__htt_global__");
+
   return APR_SUCCESS;
 }
 
@@ -1935,14 +1982,14 @@ static apr_status_t global_END(command_t *self, global_t *global, char *data,
       return APR_EINVAL;
     }
     /* get number of concurrent default is 1 */
-    val = apr_strtok(global->worker->additional, " ", &last);
+    val = apr_strtok(global->cur_worker->additional, " ", &last);
     if (val) {
       concurrent = apr_atoi64(val);
       if (concurrent <= 0) {
 	fprintf(stderr, "\nNumber of concurrent clients must be > 0");
 	return EINVAL;
       }
-      global->worker->additional = NULL;
+      global->cur_worker->additional = NULL;
     }
     else {
       concurrent = 1;
@@ -1961,8 +2008,8 @@ static apr_status_t global_END(command_t *self, global_t *global, char *data,
     break; 
   case GLOBAL_STATE_BLOCK:
     /* store block */
-    apr_hash_set(global->blocks, global->worker->name, APR_HASH_KEY_STRING, 
-	         global->worker);
+    apr_hash_set(global->blocks, global->cur_worker->name, APR_HASH_KEY_STRING, 
+	         global->cur_worker);
     global->state = GLOBAL_STATE_NONE;
     return htt_run_block_end(global);
     break; 
@@ -1977,14 +2024,14 @@ static apr_status_t global_END(command_t *self, global_t *global, char *data,
     break; 
   case GLOBAL_STATE_FILE:
     /* write file */
-    if ((status = worker_to_file(global->worker)) != APR_SUCCESS) {
-      worker_set_global_error(global->worker);
-      fprintf(stderr, "\nCould not create %s: %s(%d)", global->worker->name, 
+    if ((status = worker_to_file(global->cur_worker)) != APR_SUCCESS) {
+      worker_set_global_error(global->cur_worker);
+      fprintf(stderr, "\nCould not create %s: %s(%d)", global->cur_worker->name, 
 	      my_status_str(global->pool, status), status);
       return status;
     }
 
-    apr_pool_cleanup_register(global->pool, global->worker->name, 
+    apr_pool_cleanup_register(global->pool, global->cur_worker->name, 
                               worker_file_cleanup, apr_pool_cleanup_null);
     global->state = GLOBAL_STATE_NONE;
     return APR_SUCCESS;
@@ -1996,32 +2043,32 @@ static apr_status_t global_END(command_t *self, global_t *global, char *data,
   }
 
   /* store the workers to start them later */
-  global->worker->filename = global->filename;
+  global->cur_worker->filename = global->filename;
   while (concurrent) {
     clone = NULL;
     --concurrent;
     called_name = apr_psprintf(global->pool, "%s-%d", name, concurrent);
-    global->worker->name = called_name;
-    global->worker->which = concurrent;
+    global->cur_worker->name = called_name;
+    global->cur_worker->which = concurrent;
     if (concurrent) {
-      if ((status = worker_clone(&clone, global->worker)) != APR_SUCCESS) {
-	worker_log(global->worker, LOG_ERR, "Could not clone thread");
+      if ((status = worker_clone(&clone, global->cur_worker)) != APR_SUCCESS) {
+	worker_log(global->cur_worker, LOG_ERR, "Could not clone thread");
 	return APR_EINVAL;
       }
     }
 
     switch (global->state) {
     case GLOBAL_STATE_CLIENT:
-      apr_table_addn(global->clients, called_name, (char *) global->worker);
+      apr_table_addn(global->clients, called_name, (char *) global->cur_worker);
       break;
     case GLOBAL_STATE_SERVER:
-      apr_table_addn(global->servers, called_name, (char *) global->worker);
+      apr_table_addn(global->servers, called_name, (char *) global->cur_worker);
       break;
     case GLOBAL_STATE_DAEMON:
-      apr_table_addn(global->daemons, called_name, (char *) global->worker);
+      apr_table_addn(global->daemons, called_name, (char *) global->cur_worker);
       break;
     }
-    global->worker = clone;
+    global->cur_worker = clone;
   }
   /* reset */
   global->state = GLOBAL_STATE_NONE;
@@ -2044,7 +2091,7 @@ static apr_status_t global_worker(command_t *self, global_t *global, char *data,
 
   /* Client start */
   global->state = state;
-  if ((status = worker_new(&global->worker, data, global->prefix, global, 
+  if ((status = worker_new(&global->cur_worker, data, global->prefix, global, 
                            worker_interpret)) != APR_SUCCESS) {
     fprintf(stderr, "\nGlobal worker: could not create worker");
     return status;
@@ -2114,7 +2161,7 @@ static apr_status_t global_BLOCK(command_t * self, global_t * global,
   if ((status = htt_run_block_start(global, &data)) 
       == APR_ENOTIMPL) {
     /* Start a new worker */
-    if ((status = worker_new(&global->worker, data, global->prefix, global, 
+    if ((status = worker_new(&global->cur_worker, data, global->prefix, global, 
                              worker_interpret)) != APR_SUCCESS) {
       return status;
     }
@@ -2133,7 +2180,7 @@ static apr_status_t global_BLOCK(command_t * self, global_t * global,
       fprintf(stderr, "\nChar ':' is not allowed in block name \"%s\"", token);
       return APR_EINVAL;
     }
-    global->worker->name = data;
+    global->cur_worker->name = data;
   }
   while (token) {
     if (strcmp(token, ":") == 0) {
@@ -2142,11 +2189,11 @@ static apr_status_t global_BLOCK(command_t * self, global_t * global,
     }
     else {
       if (input) {
-       store_set(global->worker->params, apr_itoa(global->worker->pbody, i), 
+       store_set(global->cur_worker->params, apr_itoa(global->cur_worker->pbody, i), 
                   token);
       }
       else {
-        store_set(global->worker->retvars, apr_itoa(global->worker->pbody, i),
+        store_set(global->cur_worker->retvars, apr_itoa(global->cur_worker->pbody, i),
                   token);
       }
       i++;
@@ -2176,13 +2223,13 @@ static apr_status_t global_FILE(command_t * self, global_t * global,
   global->state = GLOBAL_STATE_FILE;
 
   /* Start a new worker */
-  if ((status = worker_new(&global->worker, data, global->prefix, global, 
+  if ((status = worker_new(&global->cur_worker, data, global->prefix, global, 
                            worker_interpret)) != APR_SUCCESS) {
     fprintf(stderr, "\nCould not initiate worker instance");
     return status;
   }
 
-  global->worker->name = data;
+  global->cur_worker->name = data;
 
   /* open file */
   return APR_SUCCESS;
@@ -2558,7 +2605,7 @@ static apr_status_t global_PROCESS(command_t *self, global_t *global, char *data
 #endif
 
 /**
- * Global GO command
+ * Global START command starts all so far defined threads 
  *
  * @param self IN command
  * @param global IN global object
@@ -2566,8 +2613,8 @@ static apr_status_t global_PROCESS(command_t *self, global_t *global, char *data
  *
  * @return APR_SUCCESS
  */
-static apr_status_t global_GO(command_t *self, global_t *global, char *data, 
-                              apr_pool_t *ptmp) {
+static apr_status_t global_START(command_t *self, global_t *global, char *data, 
+                                 apr_pool_t *ptmp) {
   apr_status_t status;
   apr_table_entry_t *e;
   int i;
@@ -2575,7 +2622,7 @@ static apr_status_t global_GO(command_t *self, global_t *global, char *data,
   apr_thread_t *thread;
 
 
-  /* start all daemons first */
+  /* create all daemons first */
   e = (apr_table_entry_t *) apr_table_elts(global->daemons)->elts;
   for (i = 0; i < apr_table_elts(global->daemons)->nelts; ++i) {
     worker = (void *)e[i].val;
@@ -2587,7 +2634,7 @@ static apr_status_t global_GO(command_t *self, global_t *global, char *data,
     }
   }
   apr_table_clear(global->daemons);
-  /* start all servers */
+  /* create all servers */
   e = (apr_table_entry_t *) apr_table_elts(global->servers)->elts;
   for (i = 0; i < apr_table_elts(global->servers)->nelts; ++i) {
     sync_lock(global->sync);
@@ -2602,7 +2649,7 @@ static apr_status_t global_GO(command_t *self, global_t *global, char *data,
   }
   apr_table_clear(global->servers);
 
-  /* start all clients */
+  /* create clients */
   sync_lock(global->sync);
   sync_unlock(global->sync);
   e = (apr_table_entry_t *) apr_table_elts(global->clients)->elts;
@@ -2626,18 +2673,36 @@ static apr_status_t global_GO(command_t *self, global_t *global, char *data,
   }
   apr_table_clear(global->clients);
    
-  /* start clients */
+  /* notify start threads */
   e = (apr_table_entry_t *) apr_table_elts(global->threads)->elts;
   for (i = 0; i < apr_table_elts(global->threads)->nelts; ++i) {
     thread = (apr_thread_t *) e[i].val;
     status = htt_run_thread_start(global, thread);
     if (status != APR_SUCCESS) {
-      fprintf(stderr, "\nCould not start client thread: %d", status);
+      fprintf(stderr, "\nCould not start thread: %d", status);
       return status;
     }
   }
- 
-  /* wait on thermination of all started threads */
+  return APR_SUCCESS;
+}
+
+/**
+ * Global JOIN command waits for all started threads except DAEMONs
+ *
+ * @param self IN command
+ * @param global IN global object
+ * @param data IN unused
+ *
+ * @return APR_SUCCESS
+ */
+static apr_status_t global_JOIN(command_t *self, global_t *global, char *data, 
+                                apr_pool_t *ptmp) {
+  apr_status_t status;
+  apr_table_entry_t *e;
+  int i;
+  apr_thread_t *thread;
+
+  /* join all started threads */
   e = (apr_table_entry_t *) apr_table_elts(global->threads)->elts;
   for (i = 0; i < apr_table_elts(global->threads)->nelts; ++i) {
     apr_status_t retstat;
@@ -2660,6 +2725,26 @@ static apr_status_t global_GO(command_t *self, global_t *global, char *data,
   global->prefix = apr_pstrdup(global->pool, "");
   return APR_SUCCESS;
 }
+ 
+
+/**
+ * Global GO command start all threads which are declared so far and wait
+ * until all threads except DAEMON threads do have terminated.
+ *
+ * @param self IN command
+ * @param global IN global object
+ * @param data IN unused
+ *
+ * @return APR_SUCCESS
+ */
+static apr_status_t global_GO(command_t *self, global_t *global, char *data, 
+                              apr_pool_t *ptmp) {
+  apr_status_t status = global_START(self, global, data, ptmp);
+  if (status == APR_SUCCESS) {
+    status = global_JOIN(self, global, data, ptmp);
+  }
+  return status;
+}
 
 /**
  * Recursiv interpreter. Can handle recursiv calls to with sub files i.e. INCLUDE.
@@ -2718,7 +2803,7 @@ static apr_status_t interpret_recursiv(apr_file_t *fp, global_t *global) {
 	    return status;
 	  }
         }
-        else if ((status = worker_add_line(global->worker, 
+        else if ((status = worker_add_line(global->cur_worker, 
 					   apr_psprintf(global->pool, "%s:%d", 
 					   global->filename, line_nr), line)) 
 	    != APR_SUCCESS) {
@@ -2734,7 +2819,6 @@ static apr_status_t interpret_recursiv(apr_file_t *fp, global_t *global) {
 	/* lookup function index */
 	i = 0;
 	command = lookup_command(global_commands, line);
-	/* found command? */
 	if (command->func) {
 	  i += strlen(command->name);
 	  if ((status = command->func(command, global, &line[i], NULL)) 
@@ -2742,6 +2826,19 @@ static apr_status_t interpret_recursiv(apr_file_t *fp, global_t *global) {
 	    return status;
 	  }
 	}
+        else { /* let's see if we find block for this job */
+          int cur_log_mode = global->worker->log_mode;
+          apr_pool_t *ptmp;
+
+          apr_pool_create(&ptmp, NULL);
+          global->worker->log_mode = 0;
+          status = command_CALL(NULL, global->worker, line, ptmp);
+          global->worker->log_mode = cur_log_mode;
+          apr_pool_destroy(ptmp);
+          if (status != APR_SUCCESS && !APR_STATUS_IS_ENOENT(status)) {
+            return status;
+          }
+        }
       }
     }
   }
@@ -2974,9 +3071,15 @@ static void show_commands(apr_pool_t *p, global_t *global) {
 	for (hi = apr_hash_first(p, block); hi; hi = apr_hash_next(hi)) {
 	  apr_hash_this(hi, (const void **)&command, NULL, (void **)&worker);
 	  if (command) {
-	    ++command; /* skip "_" */
-	    line = apr_psprintf(p, "_%s:%s %s", module, command, 
-		    worker->short_desc?worker->short_desc:"");
+            if (*command == '_') {
+              ++command; /* skip "_" */
+              line = apr_psprintf(p, "_%s:%s %s", module, command, 
+                      worker->short_desc?worker->short_desc:"");
+            }
+            else {
+              line = apr_psprintf(p, "%s:%s %s", module, command, 
+                      worker->short_desc?worker->short_desc:"");
+            }
 	    SKM_sk_push(char, sorted, line);
 	  }
 	}
@@ -3042,16 +3145,19 @@ static void show_command_help(apr_pool_t *p, global_t *global,
     copy = apr_pstrdup(p, command);
     /* determine module if any */
     module = apr_strtok(copy, ":", &last);
-    /* always jump over prefixing "_" */
-    module++;
-    block_name = apr_pstrcat(p, "_", last, NULL);
+    if (*module == '_') {
+      module++;
+      block_name = apr_pstrcat(p, "_", last, NULL);
+    }
+    else {
+      block_name = apr_pstrdup(p, last);
+    }
     if (!(blocks = apr_hash_get(global->modules, module, APR_HASH_KEY_STRING))) {
       fprintf(stdout, "\ncommand: %s do not exist\n\n", command);
       exit(1);
     }
-    block_name = apr_pstrcat(p, "_", last, NULL);
     if (!(worker = apr_hash_get(blocks, block_name, APR_HASH_KEY_STRING))) {
-      fprintf(stdout, "\ncommand: %s do not exist\n\n", command);
+      fprintf(stdout, "\ncommand: %s do not exist\n", command);
       exit(1);
     }
     else {
