@@ -73,16 +73,27 @@ typedef struct perf_host_s {
   apr_thread_mutex_t *sync;
   socket_t *socket; 
   worker_t *worker;
+  int flags;
+#define PERF_HOST_FLAGS_NONE 0
+#define PERF_HOST_FLAGS_GLOBALS_DIST 1
 } perf_host_t;
 
+typedef struct perf_rampup_s {
+  apr_size_t cur_clients;
+  apr_size_t clients;
+  apr_time_t interval;
+} perf_rampup_t;
+
 typedef struct perf_gconf_s {
   int on;
 #define PERF_GCONF_OFF  0
 #define PERF_GCONF_ON   1
 #define PERF_GCONF_LOG  2 
   int flags;
-#define PERF_GCONF_FLAGS_NONE 0 
-#define PERF_GCONF_FLAGS_DIST 1 
+#define PERF_GCONF_FLAGS_NONE   0 
+#define PERF_GCONF_FLAGS_DIST   1 
+#define PERF_GCONF_FLAGS_RAMPUP 2 
+  perf_rampup_t rampup;
   apr_file_t *log_file;
   apr_hash_t *my_threads;
   apr_hash_t *host_and_ports;
@@ -132,57 +143,6 @@ static perf_wconf_t *perf_get_worker_config(worker_t *worker) {
   return config;
 }
 
-/**
- * Test if statistic is turned on 
- * @param global IN global config
- * @param line IN read line
- */
-static apr_status_t perf_read_line(global_t *global, char **line) {
-
-  if (strncmp(*line, "PERF:STAT", 9) == 0) {
-    perf_gconf_t *gconf = perf_get_global_config(global);
-    char *cur;
-    char *last;
-
-    apr_strtok(*line, " ", &last);
-    cur = apr_strtok(NULL, " ", &last);
-    if (strcmp(cur, "ON") == 0) {
-      gconf->on = PERF_GCONF_ON;
-    }
-    else if (strcmp(cur, "OFF") == 0) {
-      gconf->on |= PERF_GCONF_OFF;
-    }
-    else if (strcmp(cur, "LOG") == 0) {
-      apr_status_t status;
-      char *filename;
-      gconf->on |= PERF_GCONF_LOG;
-      filename = apr_strtok(NULL, " ", &last);
-      if ((status = apr_file_open(&gconf->log_file, filename, 
-                                  APR_READ|APR_WRITE|APR_CREATE|APR_APPEND|APR_XTHREAD, 
-                                  APR_OS_DEFAULT, global->pool)) != APR_SUCCESS) {
-        fprintf(stderr, "Could not open log file \"%s\"", filename);
-        return status;
-      }
-    }
-  }
-  else if (strncmp(*line, "PERF:DISTRIBUTE", 9) == 0) {
-    char *last;
-    char *name;
-    perf_host_t *host = apr_pcalloc(global->pool, sizeof(*host));
-    perf_gconf_t *gconf = perf_get_global_config(global);
-    gconf->flags |= PERF_GCONF_FLAGS_DIST;
-    apr_strtok(*line, " ", &last);
-    name = apr_strtok(NULL, " ", &last);
-    while (name) {
-      host->name = name;
-      apr_hash_set(gconf->host_and_ports, name, APR_HASH_KEY_STRING, host);
-      host = apr_pcalloc(global->pool, sizeof(*host));
-      name = apr_strtok(NULL, " ", &last);
-    }
-  }
-  return APR_SUCCESS;
-}
-
 /**
  * Is called after line is sent
  * @param worker IN callee
@@ -335,8 +295,9 @@ static apr_status_t perf_WAIT_end(worker_t *worker, apr_status_t status) {
     apr_pool_create(&pool, NULL);
     date_str = apr_palloc(pool, APR_RFC822_DATE_LEN);
     apr_rfc822_date(date_str, apr_time_now());
-    apr_file_printf(gconf->log_file, "[%s] \"%s\" %d %"APR_TIME_T_FMT" %"APR_TIME_T_FMT"\n", 
+    apr_file_printf(gconf->log_file, "[%s] \"%s\" %d %s %"APR_TIME_T_FMT" %"APR_TIME_T_FMT"\n", 
                     date_str,  wconf->request_line, wconf->cur_status, 
+                    status == APR_SUCCESS ? "OK" : "FAILED",
                     wconf->stat.sent_time.cur, wconf->stat.recv_time.cur);
     apr_pool_destroy(pool);
   }
@@ -555,20 +516,23 @@ static apr_status_t perf_serialize_globals(global_t *global, perf_host_t *host)
   apr_table_entry_t *e;
   apr_pool_t *ptmp;
 
-  apr_pool_create(&ptmp, NULL);
-  vars = store_get_table(global->vars, ptmp);
-  e = (apr_table_entry_t *) apr_table_elts(vars)->elts;
-  for (i = 0; i < apr_table_elts(vars)->nelts; ++i) {
-    perf_serialize(host, "SET %s=%s\n", e[i].key, e[i].val);
-  }
-  if (global->shared) {
-    shared = store_get_table(global->shared, ptmp);
-    e = (apr_table_entry_t *) apr_table_elts(shared)->elts;
-    for (i = 0; i < apr_table_elts(shared)->nelts; ++i) {
-      perf_serialize(host, "GLOBAL %s=%s\n", e[i].key, e[i].val);
+  if (!host->flags & PERF_HOST_FLAGS_GLOBALS_DIST) {
+    apr_pool_create(&ptmp, NULL);
+    vars = store_get_table(global->vars, ptmp);
+    e = (apr_table_entry_t *) apr_table_elts(vars)->elts;
+    for (i = 0; i < apr_table_elts(vars)->nelts; ++i) {
+      perf_serialize(host, "SET %s=%s\n", e[i].key, e[i].val);
     }
+    if (global->shared) {
+      shared = store_get_table(global->shared, ptmp);
+      e = (apr_table_entry_t *) apr_table_elts(shared)->elts;
+      for (i = 0; i < apr_table_elts(shared)->nelts; ++i) {
+        perf_serialize(host, "GLOBAL %s=%s\n", e[i].key, e[i].val);
+      }
+    }
+    apr_pool_destroy(ptmp);
+    host->flags |= PERF_HOST_FLAGS_GLOBALS_DIST; 
   }
-  apr_pool_destroy(ptmp);
   return APR_SUCCESS;
 }
 
@@ -583,14 +547,17 @@ static apr_status_t perf_serialize_clients(global_t *global, perf_host_t *host)
   apr_pool_t *ptmp;
   apr_table_entry_t *e;
 
-  apr_pool_create(&ptmp, NULL);
-  perf_serialize(host, "CLIENT %d\n", host->clients);
-  e = (apr_table_entry_t *) apr_table_elts(host->worker->lines)->elts;
-  for (i = 0; i < apr_table_elts(host->worker->lines)->nelts; ++i) {
-    perf_serialize(host, "%s\n", e[i].val);
+  if (host->clients > 0) {
+    apr_pool_create(&ptmp, NULL);
+    perf_serialize(host, "CLIENT %d\n", host->clients);
+    e = (apr_table_entry_t *) apr_table_elts(host->worker->lines)->elts;
+    for (i = 0; i < apr_table_elts(host->worker->lines)->nelts; ++i) {
+      perf_serialize(host, "%s\n", e[i].val);
+    }
+    perf_serialize(host, "END\n");
+    apr_pool_destroy(ptmp);
+    host->clients = 0;
   }
-  perf_serialize(host, "END\n");
-  apr_pool_destroy(ptmp);
   return APR_SUCCESS;
 }
 
@@ -712,30 +679,44 @@ static apr_status_t perf_distribute_host(worker_t *worker, apr_thread_t **thread
 static apr_status_t perf_client_create(worker_t *worker, apr_thread_start_t func, apr_thread_t **new_thread) {
   global_t *global = worker->global;
   perf_gconf_t *gconf = perf_get_global_config(global);
+  apr_status_t status = APR_ENOTHREAD;
   
   if (gconf->flags & PERF_GCONF_FLAGS_DIST) {
     if (!gconf->cur_host_i) {
       worker_log(worker, LOG_INFO, "Distribute CLIENT to my self");
       perf_get_first_host(global, worker);
-      return APR_ENOTHREAD;
+      status = APR_ENOTHREAD;
     }
     else {
-      apr_status_t status;
       /* distribute to remote host */
       worker_log(worker, LOG_INFO, "Distribute CLIENT to %s", gconf->cur_host->name);
       status = perf_distribute_host(worker, new_thread);
       perf_get_next_host(global, worker);
-      if (status == APR_SUCCESS) {
-        return APR_SUCCESS;
-        /* return APR_ENOTHREAD; */
-      }
-      else {
-        return APR_ENOTHREAD;
+      if (status != APR_SUCCESS) {
+        status = APR_ENOTHREAD;
       }
     }
   }
 
-  return APR_ENOTHREAD;
+  if (gconf->flags & PERF_GCONF_FLAGS_RAMPUP) {
+    if (gconf->rampup.cur_clients >= gconf->rampup.clients) {
+      if (gconf->flags & PERF_GCONF_FLAGS_DIST) {
+        perf_host_t *host = perf_get_cur_host(gconf, worker);
+        if (host && host->state == PERF_HOST_CONNECTED) {
+          perf_serialize_globals(global, host);
+          perf_serialize_clients(global, host);
+          perf_serialize(host, "START\n");
+        }
+      }
+      apr_sleep(gconf->rampup.interval);
+      gconf->rampup.cur_clients = 0;
+    } 
+    else {
+      ++gconf->rampup.cur_clients;
+    }
+  }
+
+  return status;
 }
 
 /**
@@ -762,11 +743,143 @@ static apr_status_t perf_thread_start(global_t *global, apr_thread_t *thread) {
  * Commands 
  ***********************************************************************/
 
+/**
+ * PERF:DISTRIBUTE command
+ * @param worker IN thread data object
+ * @param data IN
+ * @return APR_SUCCESS or APR_EINVAL
+ */
+static apr_status_t block_PERF_DISTRIBUTE(worker_t * worker, worker_t *parent,
+                                          apr_pool_t *ptmp) {
+  apr_status_t status;
+  global_t *global = worker->global;
+  perf_host_t *host = apr_pcalloc(global->pool, sizeof(*host));
+  perf_gconf_t *gconf = perf_get_global_config(global);
+
+  if ((status = module_check_global(worker)) != APR_SUCCESS) {
+    return status;
+  }
+  gconf->flags |= PERF_GCONF_FLAGS_DIST;
+  host->name = store_get_copy(worker->params, global->pool, "1");
+  apr_hash_set(gconf->host_and_ports, host->name, APR_HASH_KEY_STRING, host);
+  return APR_SUCCESS;
+}
+
+/**
+ * PERF:LOG command
+ * @param worker IN thread data object
+ * @param data IN
+ * @return APR_SUCCESS or APR_EINVAL
+ */
+static apr_status_t block_PERF_STAT(worker_t * worker, worker_t *parent,
+                                    apr_pool_t *ptmp) {
+  apr_status_t status;
+  global_t *global = worker->global;
+  perf_host_t *host = apr_pcalloc(global->pool, sizeof(*host));
+  perf_gconf_t *gconf = perf_get_global_config(global);
+  const char *param;
+
+  if ((status = module_check_global(worker)) != APR_SUCCESS) {
+    return status;
+  }
+  param = store_get(worker->params, "1");
+  if (strcmp(param, "ON") == 0) {
+    gconf->on = PERF_GCONF_ON;
+  }
+  else if (strcmp(param, "OFF") == 0) {
+    gconf->on |= PERF_GCONF_OFF;
+  }
+  else if (strcmp(param, "LOG") == 0) {
+    apr_status_t status;
+    const char *filename;
+    gconf->on |= PERF_GCONF_LOG;
+    filename = store_get(worker->params, "2");
+    if (filename) {
+      if ((status = apr_file_open(&gconf->log_file, filename, 
+                                  APR_READ|APR_WRITE|APR_CREATE|APR_APPEND|APR_XTHREAD, 
+                                  APR_OS_DEFAULT, global->pool)) != APR_SUCCESS) {
+        worker_log_error(worker, "Could not open log file \"%s\"", filename);
+        return status;
+      }
+    }
+    else {
+      worker_log_error(worker, "No file specified for PERF:LOG command");
+      return APR_EINVAL;
+    }
+  }
+
+  return APR_SUCCESS;
+}
+
+/**
+ * PERF:RAMPUP command
+ * @param worker IN thread data object
+ * @param data IN
+ * @return APR_SUCCESS or APR_EINVAL
+ */
+static apr_status_t block_PERF_RAMPUP(worker_t * worker, worker_t *parent,
+                                      apr_pool_t *ptmp) {
+  apr_status_t status;
+  global_t *global = worker->global;
+  perf_host_t *host = apr_pcalloc(global->pool, sizeof(*host));
+  perf_gconf_t *gconf = perf_get_global_config(global);
+  const char *clients_str;
+  const char *interval_str;
+
+  status = APR_SUCCESS;
+  if ((status = module_check_global(worker)) == APR_SUCCESS) {
+    clients_str = store_get(worker->params, "1");
+    interval_str = store_get(worker->params, "2");
+    if (clients_str && interval_str) {
+      gconf->flags |= PERF_GCONF_FLAGS_RAMPUP;
+      gconf->rampup.clients = apr_atoi64(clients_str);
+      /* apr_time_from_msec available in apr 1.4.x */
+      gconf->rampup.interval = 1000 * apr_atoi64(interval_str);
+    }
+    else if (!clients_str) {
+      worker_log_error(worker, "Number of clients per interval not specified");
+      status = APR_ENOENT;
+    }
+    else {
+      worker_log_error(worker, "Interval not specified");
+      status = APR_ENOENT;
+    }
+  }
+
+  return status;
+}
+
 /************************************************************************
  * Module
  ***********************************************************************/
 apr_status_t perf_module_init(global_t *global) {
-  htt_hook_read_line(perf_read_line, NULL, NULL, 0);
+  apr_status_t status;
+
+  if ((status = module_command_new(global, "PERF", "STAT", 
+                                   "ON|OFF|LOG <filename>",
+				   "print statistics at end of test, option LOG "
+                                   "do additional write all requests to <filename>",
+	                           block_PERF_STAT)) != APR_SUCCESS) {
+    return status;
+  }
+
+  if ((status = module_command_new(global, "PERF", "RAMPUP", 
+                                   "<clients> per <interval>",
+				   "Start <clients> per <interval> [ms], "
+                                   "<clients> per <interval> are started all "
+                                   "together",
+	                           block_PERF_RAMPUP)) != APR_SUCCESS) {
+    return status;
+  }
+
+  if ((status = module_command_new(global, "PERF", "DISTRIBUTED", 
+                                   "<host>:<port>",
+				   "Distribute CLIENT to <host>:<port>, "
+                                   "need an agent on this host",
+	                           block_PERF_DISTRIBUTE)) != APR_SUCCESS) {
+    return status;
+  }
+
   htt_hook_client_create(perf_client_create, NULL, NULL, 0);
   htt_hook_thread_start(perf_thread_start, NULL, NULL, 0);
   htt_hook_worker_joined(perf_worker_joined, NULL, NULL, 0);
