@@ -1,8 +1,9 @@
-/* Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements. 
- * The ASF licenses this file to You under the Apache License, Version 2.0
- * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
+/**
+ * Copyright 2006 Christian Liesch
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -326,7 +327,7 @@ static apr_status_t transfer_enc_reader_bb(sockreader_t *self,
   int chunk;
   char *end;
   char *line;
-  char *read;
+  char *read = NULL;
   apr_size_t chunk_cur;
   apr_size_t chunk_len;
   apr_bucket *b;
@@ -449,6 +450,9 @@ apr_status_t eof_reader(sockreader_t * self, char **buf,
   *buf = NULL;
   (*len) = 0;
 
+  if (my_strcasestr(val, "upgrade")) {
+    return APR_SUCCESS;
+  }
   if (!my_strcasestr(val, "close")) {
     return APR_ENOTIMPL;
   }
