@@ -77,6 +77,8 @@ struct worker_s {
   interpret_f interpret;
   /* worker config */
   apr_hash_t *config;
+  /* worker block if this is a CALL */
+  worker_t *block;
   /* this is the pool where the structure lives */
   apr_pool_t *heartbeat;
   /* dies on END */
@@ -196,6 +198,7 @@ struct global_s {
   int socktmo;
   char *prefix;
   worker_t *worker;
+  worker_t *cur_worker;
   apr_threadattr_t *tattr;
   int recursiv;
 };
