@@ -7,5 +7,5 @@ echo
 echo "test_store_SOURCES=test_store.c \$(top_srcdir)/src/store.c"
 echo "AM_CFLAGS=-I\$(top_srcdir)/src"
 echo "check_PROGRAMS=test_store"
-echo "TESTS = test_store run_htt_test.sh run_htt_shell_test.sh run_hte_test.sh run_visual_test.sh run_ntlm_test.sh run_coredump_test.sh"
+echo "TESTS = test_store test_run_all.sh test_run_errors.sh test_run_visual.sh test_run_ntlm.sh test_check_coredumps.sh"
 
