@@ -1,22 +1,29 @@
 #!/bin/bash
 
-HTT_ERRORS=0
+if [ -z $srcdir ]; then
+  srcdir=.
+fi
+
+. $srcdir/run_lib.sh
+
+function run_single {
+  E=$1
+  OUT=$2
 
-for E in `ls *.visual`; do
   B=`echo $E | sed -e 's/\(.*\)\.visual/\1/'`
   if [ -f $B.visual ]; then
     ./run.sh $B.htt >/tmp/tmp.txt 2>/dev/null
     lines=`wc -l $B.visual | awk '{ print $1 }'`
     tail -n ${lines} /tmp/tmp.txt >/tmp/tmp2.txt
     diff /tmp/tmp2.txt $B.visual
-    ret=$?
-    if [ $ret -ne 0 ]; then
-      HTT_ERRORS=`expr $HTT_ERRORS + 1`
-      echo $B FAILED
-    else
-      echo $B OK
-    fi
+  else
+    printf "SKIP"
   fi
-done
+}
+
+echo visual tests
+LIST=`ls *.visual`
+COUNT=`ls *.visual | wc -l`
+run_all "$LIST" $COUNT
+
 
-exit $HTT_ERRORS
